export libdir="$(builtin cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
PATH="$libdir/../macros.d/scripts:$PATH"

. $libdir/../macros.d/helpers/utils.bash
  
function die {
    echo fail.
    exit 1
}

function skip_test {
    echo skipped.
    exit 0
}

function log_filter {
    grep -Ev -e '^> (env|cmd):' -e 'Wrote: /tmp/' -e 'git_(dir_|cwd_)?(archive|pack): archiving /tmp/'\
    -e 'git_(dir_|cwd_)?pack: packing path /tmp/' -e 'fatal: [nN]ot a git repository' -e '(GIT_DISCOVERY_ACROSS_FILESYSTEM not set)'\
    -e '^Date:' -e '^commit '
}

function run {
    echo '>' $@;
    eval $@;
}

function dirty_sub {
    latest_ctime="$(builtin cd "$1"; . $libdir/../macros.d/git.bash; utils_encode_decimal "$(git_latest_ctime)")"
    sed -Ei "s/\.dirty\.[a-zA-Z0-9]+/.dirty.$latest_ctime/" "$2"
}

function head_commit_sub {
    full_hash=$(git -C "$1" rev-parse HEAD)
    short_hash=$(git -C "$1" rev-parse --short HEAD)
    sed -Ei -e "s|(\.git\.[0-9]+)\.[a-zA-Z0-9]{7}|\1.$short_hash|" -e "s|#\w{40}:|#$full_hash:|" \
        -e "s|-[a-zA-Z0-9]{7}(-dirty)?.tar.gz$|-$short_hash\1.tar.gz|" -e "s|(setup -q -n .+)-[a-zA-Z0-9]{7}(-dirty)?$|\1-$short_hash\2|" "$2"
}

function xsubs {
    sed -Ei "s/\.dirty\.[a-zA-Z0-9]+/.dirty.xxxxxx/" "$1"
    sed -Ei -e "s|(\.git\.[0-9]+)\.[a-zA-Z0-9]+|\1.xxxxxxx|" -e "s|#\w{40}:|#xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx:|" "$1"
}

function prepare_input {
    cat "$1" | sed -Ee 's|^(.+) ::$|\1:\n{{{ \1 }}}\n|' | sed -Ee 's|^(.+) :$|\1:\n{{{ \1 }}}|' | sed -Ee 's|^(.*[^:}])$|\1: {{{ \1 }}}|' > "$2"
}

function generic_test {
    declare preproc_opts= outdir= input= expected_output= expected_log= custom_test= copy_only= "$@"

    run "cat $input | preproc $preproc_opts -e PATH=$libdir/../bin:/usr/bin:/bin -e INPUT_DIR_PATH='$(dirname $input)' -s $libdir/../macros.d/all.bash --debug --no-break-on-failure 2>&1 > $outdir/output | log_filter > $outdir/log"

    if [ -n "$copy_only" ]; then
        cp "$outdir/log" "$outdir/output" .
        echo copied.
        rm -r "$outdir"
        exit
    fi

    run "diff $expected_output $outdir/output || { echo 'LOG DIFF:' && diff $expected_log $outdir/log && die; }"
    run "diff $expected_log $outdir/log || die"

    if [ "$custom_test" ]; then
        run "$custom_test || die"
    fi

    echo success.
}
