import logging
import os
import re
import textwrap
import subprocess
import tempfile

from .errors import RpkgError

log = logging.getLogger("__main__")


def get_changelog_entry(commit_message):
    entry = '- ' + commit_message.split('\n')[0] if commit_message else ''
    return '\n'.join(textwrap.wrap(re.sub(r'(\s+)%([^%])', r'\1%%\2', entry), 80))


def run(cmd, shell=False, env=None, cwd=None, capture=True, capture_stderr=False, throw=True):
    if capture:
        stdout = subprocess.PIPE
    else:
        stdout = None

    if capture_stderr:
        stderr = subprocess.PIPE
    else:
        stderr = None

    process = subprocess.Popen(
        cmd, stdout=stdout, stderr=stderr, shell=shell, env=env, cwd=cwd)

    (stdout, stderr) = process.communicate()

    if process.returncode != 0 and throw:
        raise RpkgError(stderr)

    if stdout:
        return stdout.decode('utf-8').strip()

    return ''


def edit(editor, text):
    tmp_f = tempfile.NamedTemporaryFile('w', delete=False)
    tmp_f.write(text)
    tmp_f.close()
    subprocess.check_call(editor.split()+[tmp_f.name])
    tmp_f = open(tmp_f.name, 'r')
    result = tmp_f.read()
    tmp_f.close()
    os.unlink(tmp_f.name)
    return result
