#!/usr/bin/python

import cgi
import errno
import grp
import hashlib
import os
import sys
import tempfile

from configparser import ConfigParser

# Reading buffer size
BUFFER_SIZE = 4096


def send_error(text, status='500 Internal Server Error'):
    print 'Status: %s' % status
    print 'Content-type: text/plain'
    print
    print text
    sys.exit(0)


def check_form(form, var):
    sys.stderr.write('wtf')
    ret = form.getvalue(var, None)
    sys.stderr.write('wtf2')
    if ret is None:
        send_error('Required field "%s" is not present.' % var,
                   status='400 Bad Request')
    if isinstance(ret, list):
        send_error('Multiple values given for "%s". Aborting.' % var,
                   status='400 Bad Request')
    return ret


def makedirs(dir_path, mode=02755):
    try:
        os.makedirs(dir_path, mode=mode)
        sys.stderr.write('mkdir %s\n' % dir_path)
    except OSError as e:
        if e.errno != errno.EEXIST:
            send_error(str(e))


def ensure_namespaced(name, namespace):
    if not namespace:
        return name

    name_parts = name.split('/')
    if len(name_parts) == 1:
        return os.path.join(namespace, name)

    return name


def main():
    config = ConfigParser()
    config.read(os.path.join(os.path.dirname(__file__), 'upload.conf'))
    os.umask(002)

    print 'Content-Type: text/plain'
    print

    sys.stderr.write('wtf0')
    form = cgi.FieldStorage()
    name = check_form(form, 'name').strip('/')

    if 'sha512sum' in form:
        checksum = check_form(form, 'sha512sum')
        hash_type = "sha512"
    else:
        send_error('Required checksum is not present.',
                   status='400 Bad Request')

    action = None
    upload_file = None
    filename = None

    # Is this a submission or a test?
    # in a test, we don't get a file, just a filename.
    # In a submission, we don't get a filename, just the file.
    if 'filename' in form:
        action = 'check'
        filename = check_form(form, 'filename')
        filename = os.path.basename(filename)
        sys.stderr.write('Checking file status: NAME=%s '
                         'FILENAME=%s %sSUM=%s\n' % (name, filename,
                                                     hash_type.upper(),
                                                     checksum))
    else:
        action = 'upload'
        if 'file' in form:
            upload_file = form['file']
            if not upload_file.file:
                send_error('No file given for upload. Aborting.',
                           status='400 Bad Request')
            filename = os.path.basename(upload_file.filename)
        else:
            send_error('Required field "file" is not present.',
                       status='400 Bad Request')

        sys.stderr.write('Processing upload request: '
                         'NAME=%s FILENAME=%s %sSUM=%s\n' % (
                             name, filename, hash_type.upper(),
                             checksum))

    name = ensure_namespaced(name, config['dist-git'].get('default_namespace')).strip('/')
    module_dir = os.path.join(config['dist-git']['lookaside_dir'], name)
    hash_dir = os.path.join(module_dir, filename, hash_type, checksum)
    dest_file = os.path.join(hash_dir, filename)

    if os.path.exists(dest_file):
        if action == 'check':
            print 'Available'
        else:
            upload_file.file.close()
            dest_file_stat = os.stat(dest_file)
            print 'File %s already exists' % filename
            print 'File: %s Size: %d' % (dest_file, dest_file_stat.st_size)
        sys.exit(0)
    elif action == 'check':
        print 'Missing'
        sys.exit(0)

    # check that all directories are in place
    makedirs(module_dir)

    # grab a temporary filename and dump our file in there
    tempfile.tempdir = module_dir
    tmpfile = tempfile.mkstemp(checksum)[1]
    tmpfd = open(tmpfile, 'w')

    # now read the whole file in
    m = getattr(hashlib, hash_type)()
    filesize = 0
    while True:
        data = upload_file.file.read(BUFFER_SIZE)
        if not data:
            break
        tmpfd.write(data)
        m.update(data)
        filesize += len(data)

    # now we're done reading, check the checksum of what we got
    tmpfd.close()
    check_checksum = m.hexdigest()
    if checksum != check_checksum:
        os.unlink(tmpfile)
        send_error("%s check failed. Received %s instead of %s." %
                   (hash_type.upper(), check_checksum, checksum),
                   status='400 Bad Request')

    makedirs(hash_dir)
    os.rename(tmpfile, dest_file)
    os.chmod(dest_file, 0644)

    sys.stderr.write('Stored %s (%d bytes)' % (dest_file, filesize))
    print 'File %s size %d %s %s stored OK' % (filename, filesize,
                                               hash_type.upper(), checksum)


if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        import traceback
        sys.stderr.write('%s\n' % traceback.format_exc())
        send_error(str(e))
