import os
import unittest
import shutil
import rpm
import tempfile
from rpkglib import utils

from .spec_templates import SPEC_TEMPLATE

class TestCase(unittest.TestCase):
    def setUp(self):
        self.srcdir = tempfile.mkdtemp()
        self.outdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.srcdir)
        shutil.rmtree(self.outdir)

    def dump_spec(self, template, pkgname='testpkg', subdir=None, version="1",
                  release="1", source0='source0.tar.gz', **kwargs):

        spec_content = template.substitute(
            kwargs, pkgname=pkgname, version=version,
            release=release, source0=source0)

        if subdir:
            dirpath = os.path.join(self.srcdir, subdir)
            os.makedirs(dirpath)
        else:
            dirpath = self.srcdir

        spec_path = os.path.join(dirpath, '{0}.spec'.format(pkgname))
        spec_file = open(spec_path, 'w')
        spec_file.write(spec_content)
        spec_file.close()
        return spec_path

    def touch_file(self, filename, subdir=None):
        if subdir:
            dirpath = os.path.join(self.srcdir, subdir)
            os.makedirs(dirpath)
        else:
            dirpath = self.srcdir
        filepath = os.path.join(dirpath, filename)
        open(filepath, 'w').close()
        return filepath

    def get_parsed_spec(self, spec_path):
        ts = rpm.ts()
        rpm.addMacro("_sourcedir", self.srcdir)
        return ts.parseSpec(spec_path)

    def make_packed_content(self):
        spec_path = self.dump_spec(SPEC_TEMPLATE, source0='source0.tar.gz')
        self.touch_file('source0.tar.gz')

    def make_unpacked_content(self):
        spec_path = self.dump_spec(SPEC_TEMPLATE, source0='source0.tar.gz')
        self.touch_file('foobar.py')

    def init_repo_with_commit(self):
        cmds = (
            ['git', 'init'],
            ['touch', 'README'],
            ['git', 'add', '-A'],
            ['git', 'config', 'user.name', 'tester'],
            ['git', 'config', 'user.email', 'tester@example.com'],
            ['git', 'commit', '-m', '"Add README"'],
        )
        for cmd in cmds:
            utils.run_command(cmd, cwd=self.srcdir)
